@@ -151,12 +151,14 @@ async fn test_build_request(http_engine: HttpEngine) {
 }
 
 /// Test building just a URL. Should include query params, but headers/body
-/// should *not* be built
+/// should *not* be built. Include inline query param and fragment; they should
+/// not be dropped
 #[rstest]
 #[tokio::test]
 async fn test_build_url(http_engine: HttpEngine) {
     let recipe = Recipe {
-        url: "{{ host }}/users/{{ user_id }}".into(),
+        // query param and fragment should be retained
+        url: "{{ host }}/users/{{ user_id }}?inline=1#fragment".into(),
         query: indexmap! {
             "mode".into() => ["{{ mode }}", "user"].into(),
             "fast".into() => ["true", "false"].into(),
@@ -169,7 +171,9 @@ async fn test_build_url(http_engine: HttpEngine) {
 
     assert_eq!(
         url.as_str(),
-        "http://localhost/users/1?mode=sudo&mode=user&fast=true&fast=false"
+        "http://localhost/users/1\
+            ?inline=1&mode=sudo&mode=user&fast=true&fast=false\
+            #fragment"
     );
 }
 
@@ -525,6 +529,25 @@ async fn test_body_stream(
     assert_eq!(body, expected_body, "Incorrect body");
 }
 
+/// Test overriding URL in BuildOptions
+#[rstest]
+#[tokio::test]
+async fn test_override_url(http_engine: HttpEngine) {
+    let recipe = Recipe::factory(());
+    let context = template_context(recipe, None);
+
+    let seed = seed(
+        &context,
+        BuildOptions {
+            url: Some("http://custom-host/users/{{ username }}".into()),
+            ..Default::default()
+        },
+    );
+    let ticket = http_engine.build(seed, &context).await.unwrap();
+
+    assert_eq!(ticket.record.url.as_str(), "http://custom-host/users/user");
+}
+
 /// Test overriding authentication in BuildOptions
 #[rstest]
 #[tokio::test]
@@ -863,13 +886,9 @@ async fn test_profile_duplicate_error(http_engine: HttpEngine) {
     let context = template_context(recipe, None);
 
     let seed = RequestSeed::new(recipe_id, BuildOptions::default());
-    assert_err!(
-        http_engine
-            .build(seed, &context)
-            .await
-            // Include full error chain in the message
-            .map_err(anyhow::Error::from),
-        "fake_fn(): Unknown function"
+    assert_err(
+        http_engine.build(seed, &context).await,
+        "fake_fn(): Unknown function",
     );
 }
 
@@ -983,10 +1002,58 @@ async fn test_build_curl(http_engine: HttpEngine) {
     let seed = seed(&context, BuildOptions::default());
 
     let command = http_engine.build_curl(seed, &context).await.unwrap();
-    let expected_command = "curl -XGET \
-    --url 'http://localhost/url?mode=sudo&fast=true&fast=false' \
-    --header 'accept: application/json' \
-    --header 'content-type: application/json'";
+    let expected_command = r"curl -XGET --url 'http://localhost/url?mode=sudo&fast=true&fast=false' \
+  --header 'accept: application/json' \
+  --header 'content-type: application/json'";
+    assert_eq!(command, expected_command);
+}
+
+/// Build a curl command with a large JSON body to demonstrate multiline
+/// formatting. This example is taken from GitHub issue #678.
+#[rstest]
+#[tokio::test]
+async fn test_build_curl_complex_json(http_engine: HttpEngine) {
+    let recipe = Recipe {
+        method: HttpMethod::Post,
+        url: "{{ host }}/json".into(),
+        headers: indexmap! {
+            "Authorization".into() => "Bearer {{ token }}".into(),
+        },
+        body: Some(
+            RecipeBody::json(json!({
+                "fishes": [
+                    {"species": "Salmon", "name": "Sam"},
+                    {"species": "Blue Tang", "name": "Dory"},
+                    {"species": "Clownfish", "name": "Nemo"},
+                ],
+                "count": 207,
+            }))
+            .unwrap(),
+        ),
+        ..Recipe::factory(())
+    };
+    let context = template_context(recipe, None);
+    let seed = seed(&context, BuildOptions::default());
+    let command = http_engine.build_curl(seed, &context).await.unwrap();
+    let expected_command = r#"curl -XPOST --url 'http://localhost/json' \
+  --header 'authorization: Bearer tokenzzz' \
+  --json '{
+  "fishes": [
+    {
+      "species": "Salmon",
+      "name": "Sam"
+    },
+    {
+      "species": "Blue Tang",
+      "name": "Dory"
+    },
+    {
+      "species": "Clownfish",
+      "name": "Nemo"
+    }
+  ],
+  "count": 207
+}'"#;
     assert_eq!(command, expected_command);
 }
 
@@ -1024,7 +1091,8 @@ async fn test_build_curl_authentication(
     let seed = seed(&context, BuildOptions::default());
     let command = http_engine.build_curl(seed, &context).await.unwrap();
     let expected_command = format!(
-        "curl -XGET --url 'http://localhost/url' {expected_arguments}",
+        "curl -XGET --url 'http://localhost/url' \\
+  {expected_arguments}",
     );
     assert_eq!(command, expected_command);
 }
@@ -1038,14 +1106,14 @@ async fn test_build_curl_authentication(
 )]
 #[case::json(
     RecipeBody::json(json!({"group_id": "{{ group_id }}"})).unwrap(),
-    r#"--json '{"group_id":"3"}'"#
+    "--json '{\n  \"group_id\": \"3\"\n}'"
 )]
 #[case::form_urlencoded(
     RecipeBody::FormUrlencoded(indexmap! {
         "user_id".into() => "{{ user_id }}".into(),
         "token".into() => "{{ token }}".into()
     }),
-    "--data-urlencode 'user_id=1' --data-urlencode 'token=tokenzzz'"
+    "--data-urlencode 'user_id=1' \\\n  --data-urlencode 'token=tokenzzz'"
 )]
 #[case::form_multipart(
     // This doesn't support binary content because we can't pass it via cmd
@@ -1053,7 +1121,7 @@ async fn test_build_curl_authentication(
         "user_id".into() => "{{ user_id }}".into(),
         "token".into() => "{{ token }}".into()
     }),
-    "-F 'user_id=1' -F 'token=tokenzzz'"
+    "-F 'user_id=1' \\\n  -F 'token=tokenzzz'"
 )]
 #[case::form_multipart_file(
     RecipeBody::FormMultipart(indexmap! {
@@ -1085,8 +1153,11 @@ async fn test_build_curl_body(
         // Dynamic replacements for system-specific contents
         .replace('/', path::MAIN_SEPARATOR_STR)
         .replace("{ROOT}", &context.root_dir.to_string_lossy());
-    let expected_command =
-        format!("curl -XGET --url 'http://localhost/url' {expected_arguments}");
+
+    let expected_command = format!(
+        "curl -XGET --url 'http://localhost/url' \\
+  {expected_arguments}"
+    );
     assert_eq!(command, expected_command);
 }
 
