@@ -7,7 +7,7 @@
 pub mod collection;
 pub mod database;
 pub mod http;
-pub mod template;
+pub mod render;
 #[cfg(any(test, feature = "test"))]
 pub mod test_util;
 pub mod util;
