@@ -1,6 +1,6 @@
 //! Import request collections from VSCode `.rest` files or Jetbrains `.http`
-//! files. VSCode: https://github.com/Huachao/vscode-restclient
-//! Jetbrains: https://www.jetbrains.com/help/idea/http-client-in-product-code-editor.html
+//! files. VSCode: <https://github.com/Huachao/vscode-restclient>
+//! Jetbrains: <https://www.jetbrains.com/help/idea/http-client-in-product-code-editor.html>
 
 use anyhow::{Context, anyhow};
 use indexmap::IndexMap;
@@ -21,7 +21,7 @@ use rest_parser::{
     headers::Authorization as RestAuthorization,
     template::Template as RestTemplate,
 };
-use slumber_util::ResultTraced;
+use slumber_util::{ResultTracedAnyhow, yaml::SourceLocation};
 use tracing::error;
 
 use crate::ImportInput;
@@ -169,6 +169,7 @@ fn try_build_recipe(
 
     Ok(Recipe {
         id,
+        location: SourceLocation::default(),
         persist: true,
         name: name.into(),
         method,
